from sandboxcreator.input_parser.sandbox import Sandbox
from sandboxcreator.io.reader import Reader
from sandboxcreator.io.writer import Writer
from pathlib import Path


class Provision:
    """User provisioning generator"""

    @staticmethod
    def _provisioning_exists(sandbox) -> bool:
        """Check whether the provisioning directory already exists"""
        return Reader.file_exists(sandbox.sandbox_dir /
                                  sandbox.config["provisioning_playbook"])

    @staticmethod
    def _generate_template_provisioning(sandbox: Sandbox):
        """Generate an empty template playbook"""
        provisioning_path: Path = sandbox.sandbox_dir / \
                                  sandbox.config["provisioning_dir"]
        if Provision._provisioning_exists(sandbox):
            Writer.remove_directory(provisioning_path)
        Writer.copy_file(Path(__file__).parent.parent /
                         "resources/files/user_playbook.yml", sandbox.sandbox_dir /
                         sandbox.config["provisioning_playbook"])

    @staticmethod
    def _copy_user_provisioning(sandbox: Sandbox):
        """Copy user provided provisioning files to the appropriate directory"""
        provisioning_path: Path = sandbox.sandbox_dir / \
                                  sandbox.config["provisioning_dir"]
        if Provision._provisioning_exists(sandbox):
            Writer.remove_directory(provisioning_path)
        Writer.copy_directory(sandbox.user_provisioning_dir, provisioning_path)

    @staticmethod
    def generate_user_provisioning(sandbox: Sandbox):
        """Generate or copy user provisioning files"""
        if sandbox.user_provisioning_dir is not None:
            Provision._copy_user_provisioning(sandbox)
        elif sandbox.generate_provisioning or \
                not Provision._provisioning_exists(sandbox):
            Provision._generate_template_provisioning(sandbox)
        if sandbox.extra_vars is not None:
            Writer.copy_file(sandbox.extra_vars,
                             sandbox.sandbox_dir /
                             sandbox.config["user_extra_vars"])



from typing import Dict
from pathlib import Path

from sandboxcreator.ansible_generator.routes import Routes
from sandboxcreator.io.writer import Writer
from sandboxcreator.io.reader import Reader
from sandboxcreator.input_parser.sandbox import Sandbox


class Preconfig:
    """Pre-configuration generator"""

    @staticmethod
    def _create_host_vars(sandbox: Sandbox) -> Dict[str, Dict]:
        """Create variables for each host"""
        host_vars: Dict[str, Dict] = {}
        for device in sandbox.devices:
            variables: Dict = {"routes": Routes.create_routes(device, sandbox)}
            host_vars[device.name] = variables

        return host_vars

    @staticmethod
    def _create_group_vars(sandbox: Sandbox) -> Dict[str, Dict]:
        """Create variables for each group"""
        aliases: Dict[str, str] = {}
        for device in sandbox.devices:
            for interface in device.interfaces:
                aliases[str(interface.ip)] = device.name

        all_vars: Dict = {"device_aliases": aliases}
        if sandbox.border_router_present:
            all_vars["border_router_name"] = sandbox.config["border_router_name"]
        if sandbox.controller_present:
            all_vars["controller_name"] = sandbox.config["controller_name"]
        hosts_vars: Dict = {}
        router_vars: Dict = {}
        ssh_vars: Dict = {"ansible_python_interpreter": "python3"}
        winrm_vars: Dict = {"ansible_connection": "winrm",
                            "ansible_user": "windows",
                            "ansible_password": "vagrant",
                            "ansible_become_pass": "vagrant",
                            "ansible_winrm_transport": "basic",
                            "ansible_winrm_server_cert_validation": "ignore",
                            "ansible_winrm_scheme": "http"}

        if sandbox.ansible_installed:
            ssh_vars.update({"ansible_host": "127.0.0.1",
                             "ansible_user": "vagrant"})
        else:
            ssh_vars.update({"ansible_connection": "local"})

        group_vars: Dict[str, Dict] = {"all": all_vars, "hosts": hosts_vars,
                                       "routers": router_vars, "ssh": ssh_vars,
                                       "winrm": winrm_vars}
        return group_vars

    @staticmethod
    def _preconfig_exists(sandbox) -> bool:
        """Check whether the provisioning directory already exists"""
        return Reader.dir_exists(sandbox.sandbox_dir /
                                 sandbox.config["preconfig_dir"])

    @staticmethod
    def generate_preconfig(sandbox: Sandbox) -> None:
        """Generate all files for pre-configuration"""

        if Preconfig._preconfig_exists(sandbox):
            Writer.remove_directory(sandbox.sandbox_dir /
                                    sandbox.config["preconfig_dir"])

        Writer.clone_git_repository(sandbox.config["common_repo"],
                                    sandbox.sandbox_dir / sandbox.config["preconfig_roles"])
        Writer.clone_git_repository(sandbox.config["interface_repo"],
                                    sandbox.sandbox_dir / sandbox.config["preconfig_roles"])

        host_vars: Dict[str, Dict] = Preconfig._create_host_vars(sandbox)
        for host, variables in host_vars.items():
            if variables:
                Writer.generate_yaml(sandbox.sandbox_dir
                                     / sandbox.config["preconfig_host_vars"]
                                     / f"{host}.yml", variables)

        group_vars: Dict[str, Dict] = Preconfig._create_group_vars(sandbox)
        for group, variables in group_vars.items():
            if variables:
                Writer.generate_yaml(sandbox.sandbox_dir
                                     / sandbox.config["preconfig_group_vars"]
                                     / f"{group}.yml", variables)

        Writer.copy_file(Path(__file__).parent.parent /
                         "resources/files/playbook.yml", sandbox.sandbox_dir /
                         sandbox.config["preconfig_playbook"])



from enum import Enum
from pathlib import Path
import subprocess
from typing import List, Union


class Command(Enum):
    BUILD = 1
    DESTROY = 2


class Output(Enum):
    DEVNULL = 1
    STDOUT = 2


def _validate_command_name(command: Command) -> Command:
    """Validate the command name"""

    if not isinstance(command, Command):
        raise TypeError("Invalid command type")

    return command


def _process_vagrantfile_location(vagrantfile_location: Union[str, Path]) \
        -> Path:
    """Validate and resolve path to directory containing Vagrantfile"""

    try:
        if isinstance(vagrantfile_location, str):
            if vagrantfile_location == "":
                raise ValueError("Vagrantfile location cannot be empty")
            vagrantfile_dir_path: Path = Path(vagrantfile_location).resolve()
        elif isinstance(vagrantfile_location, Path):
            vagrantfile_dir_path: Path = vagrantfile_location.resolve()
        else:
            raise TypeError("Vagrantfile location has invalid type "
                            f"\"{type(vagrantfile_location)}\"")
        if not vagrantfile_dir_path.is_dir():
            raise IOError(f"Directory {vagrantfile_dir_path} does not exist")
        if not vagrantfile_dir_path.joinpath("Vagrantfile").is_file():
            raise IOError(f"Directory {vagrantfile_dir_path} does not contain Vagrantfile")
    except OSError:
        raise ValueError("Invalid path to Vagrantfile directory"
                         f"\"{vagrantfile_location}\"")

    return vagrantfile_dir_path


def _process_machines_list(machines: List[str]) -> List[str]:
    """Validate the list of machines"""

    if not isinstance(machines, List):
        raise TypeError("List of machines must be a List")
    for machine in machines:
        if not isinstance(machine, str):
            raise TypeError("Machine names must be strings")

    return machines


def _validate_cli_output(output: Output) -> Output:
    """Validate output device"""

    if not isinstance(output, Output):
        raise TypeError("Invalid output type")

    return output


def _generate_cli_command(command: Command, machines_list: List[str]) \
        -> List[str]:
    """Generate CLI command for Vagrant"""

    cli_command: List[str] = ["vagrant"]
    if command is Command.BUILD:
        cli_command.append("up")
    elif command is Command.DESTROY:
        cli_command += ["destroy", "-f"]
    cli_command += machines_list

    return cli_command


def _translate_output(output: Output):
    """Translate Output enum values to subprocess values"""

    if output is Output.DEVNULL:
        return subprocess.DEVNULL
    if output is Output.STDOUT:
        return None


def _execute_cli_command(cli_command: List[str], vagrantfile_dir: Path,
                         output: Output, error: Output):
    """Execute Vagrant CLI command"""

    subprocess.run(cli_command, cwd=vagrantfile_dir, check=True,
                   stdout=_translate_output(output),
                   stderr=_translate_output(error))


def manage(command: Command, vagrantfile_location: Union[str, Path],
           machines: List[str] = [], output: Output = Output.DEVNULL,
           error: Output = Output.DEVNULL):
    """Vagrant wrapper to simplify sandbox instantiation and manipulation.

    :param command: type of desired manipulation (build, destroy, ...)
    :param vagrantfile_location: path to a directory containing Vagrantfile
    :param machines: list of involved machine names
    :param output: where to dump the command line output (devnull, stdout, ...)
    :param error: where to dump the command line error output (stdout, ...)
    """

    command_name: Command = _validate_command_name(command)
    vagrantfile_dir: Path = _process_vagrantfile_location(vagrantfile_location)
    machines_list: List[str] = _process_machines_list(machines)
    cli_output: Output = _validate_cli_output(output)
    cli_error: Output = _validate_cli_output(error)

    cli_command: List[str] = _generate_cli_command(command_name, machines_list)

    _execute_cli_command(cli_command, vagrantfile_dir, cli_output, cli_error)



#!/usr/bin/env python3

import argparse
import os
import sys
from typing import List, Optional

from sandboxcreator import manager


def _parse_cli_args(cli_args: List[str]) -> argparse.ArgumentParser:
    """Parse command line arguments"""

    parser = argparse.ArgumentParser()

    parser.add_argument("command",
                        help="command to be executed",
                        choices=["build", "destroy"])
    parser.add_argument("-d", "--vagrantfile_directory",
                        help="path to a directory containing a Vagrantfile",
                        default=os.getcwd())
    parser.add_argument("-m", "--machines",
                        help="list of machines involved",
                        nargs='*',
                        default=[])
    parser.add_argument("-o", "--out",
                        help="command line output",
                        choices=["devnull", "stdout"],
                        default="stdout")
    parser.add_argument("-e", "--err",
                        help="command line error output",
                        choices=["devnull", "stdout"],
                        default="stdout")

    return parser.parse_args(cli_args)


def _print_message(command: manager.Command,
                   exception: Optional[Exception] = None) -> None:
    """Print final message about success or failure"""

    if exception is None:
        if command is manager.Command.BUILD:
            print("\nSandbox was successfully built")
        elif command is manager.Command.DESTROY:
            print("\nSandbox was successfully destroyed")
    else:
        if command is manager.Command.BUILD:
            print(f"\nSandbox building process has failed:\n{exception}")
        elif command is manager.Command.DESTROY:
            print(f"\nSandbox destroying process has failed:\n{exception}")


def _translate_str_command(command_str: str) -> manager.Command:
    """Translate string commands to enum"""

    if command_str == "build":
        return manager.Command.BUILD
    if command_str == "destroy":
        return manager.Command.DESTROY
    raise ValueError(f"Invalid command: {command_str}")


def _translate_str_output(output_str: str) -> manager.Output:
    """Translate string output device to enum"""

    if output_str == "devnull":
        return manager.Output.DEVNULL
    if output_str == "stdout":
        return manager.Output.STDOUT
    raise ValueError(f"Invalid output device: {output_str}")


def _run_manager(cli_args: argparse.ArgumentParser) -> None:
    try:
        command: manager.Command = _translate_str_command(cli_args.command)
        output_dev: manager.Output = _translate_str_output(cli_args.out)
        error_dev: manager.Output = _translate_str_output(cli_args.err)

        manager.manage(command, cli_args.vagrantfile_directory,
                       cli_args.machines, output_dev, error_dev)
    except Exception as error:
        _print_message(command, error)
        sys.exit(1)

    _print_message(command)


def main():
    parsed_args: argparse.ArgumentParser = _parse_cli_args(sys.argv[1:])
    _run_manager(parsed_args)


if __name__ == '__main__':
    main()



from enum import Enum
from pathlib import Path
from typing import List, Dict, Optional, Tuple

from netaddr import IPNetwork, IPAddress
import kypo.topology_definition.models as kypo

from sandboxcreator.input_parser.topology_parser import Topology
from sandboxcreator.io.reader import Reader


class Protocol(Enum):
    """VM communication protocol"""
    SSH = 1
    WINRM = 2

    def __str__(self):
        return self.name


class DeviceType(Enum):
    """Type of device"""
    ROUTER = 1
    HOST = 2

    def __str__(self):
        return self.name.lower()


class NetworkType(Enum):
    """Type of network"""
    PRIVATE = 1
    PUBLIC = 2

    def __str__(self):
        return f"{self.name.lower()}_network"


class DevicePurpose(Enum):
    """Function of a device - also determines building order"""
    BORDER_ROUTER = 1
    ROUTER = 2
    HOST = 3
    CONTROLLER = 4

    def __str__(self):
        return self.name.lower()


class Flavor:
    """Representation of a flavor"""

    def __init__(self, name: str, memory: int, cpus: int):
        self.name: str = name
        self.memory: int = memory
        self.cpus: int = cpus

    def __str__(self):
        return f"Flavor(name: {self.name}, memory: {self.memory} GB," \
               f"CPUs: {self.cpus})"


class Network:
    """Virtual network"""

    def __init__(self, name: str, network_type: NetworkType, cidr: str):
        self.name: str = name
        self.type: NetworkType = network_type
        self.cidr: IPNetwork = IPNetwork(cidr)

    def __str__(self):
        return f"Network(name: {self.name}, type: {self.type}," \
               f" cidr: {self.cidr})"


class Interface:
    """Network interface of a device"""

    def __init__(self, network: Network, ip: str):
        self.network: Network = network
        self.ip: IPAddress = IPAddress(ip)

    def __str__(self):
        return f"Interface(network: {self.network.name}, " \
               f"type: {self.network.type}, ip: {self.ip})"


class Device:
    """A device of the sandbox"""

    def __init__(self, name: str, device_purpose: DevicePurpose, box: str,
                 memory: int, cpus: int, protocol: Protocol = Protocol.SSH,
                 interfaces: List[Interface] = [],
                 usb_passthrough: bool = False):
        self.name: str = name
        self.device_purpose: DevicePurpose = device_purpose
        if (device_purpose is DevicePurpose.ROUTER or
                device_purpose == DevicePurpose.BORDER_ROUTER):
            self.device_type = DeviceType.ROUTER
        elif (device_purpose is DevicePurpose.HOST or
              device_purpose == DevicePurpose.CONTROLLER):
            self.device_type = DeviceType.HOST
        else:
            raise ValueError("Invalid device purpose")
        self.box: str = box
        self.protocol: Protocol = protocol
        self.memory: int = memory
        self.cpus: int = cpus
        self.interfaces: List[Interface] = interfaces
        self.usb_passthrough: bool = usb_passthrough

    def __lt__(self, other):
        return self.device_purpose.value < other.device_purpose.value

    def __str__(self):
        return f"Device(name: {self.name}, type: {self.device_type}, purpose: " \
               f"{self.device_purpose}, box: {self.box})"


class Sandbox:
    """Structure that holds all information about the sandbox"""

    def __init__(self, topology_path: Path, configuration_path: Path,
                 flavors_path: Path, sandbox_dir: Path, border_router: bool,
                 ansible_installed: bool, user_provisioning_dir: Optional[Path],
                 extra_vars_file: Optional[Path], generate_provisioning: bool,
                 verbose_ansible: bool):
        try:
            topology: Topology = Topology.from_file(topology_path)
        except Exception as e:
            raise ValueError(f"Invalid topology definition:\n{e}")
        self.border_router_present: bool = border_router
        self.generate_provisioning: bool = generate_provisioning
        self.sandbox_dir: Path = sandbox_dir
        self.ansible_installed: bool = ansible_installed
        self.verbose_ansible: bool = verbose_ansible
        self.user_provisioning_dir: Optional[Path] = user_provisioning_dir
        if self.user_provisioning_dir is not None:
            self.include_requirements: bool = (self.user_provisioning_dir /
                                               "requirements.yml").is_file()
        else:
            self.include_requirements: bool = False
        self.extra_vars: Optional[Path] = extra_vars_file
        self.config: Dict = Reader.open_yaml(configuration_path)
        self.flavors: List[Flavor] = self._load_flavors(flavors_path)
        self.networks: List[Network] = self._create_network_list(topology,
                                                                 border_router,
                                                                 self.config)
        self.devices: List[Device] = self._create_device_list(topology,
                                                              self.flavors,
                                                              self.config,
                                                              border_router,
                                                              ansible_installed,
                                                              self.networks)
        self.controller_present: bool =\
            Sandbox._controller_needed(self.devices, ansible_installed)
        self.groups: Dict = Sandbox._load_groups(topology)

    @classmethod
    def _load_groups(cls, topology: Topology) -> Dict:
        """Load the list of user defined Ansible groups"""
        groups: Dict = {}

        for group in topology.groups:
            groups[group.name] = group.nodes

        return groups

    @classmethod
    def _create_network_list(cls, topology: Topology, border_router: bool,
                             config: Dict) -> List[Network]:
        """Create list of networks"""
        networks: List[Network] = []
        for network in topology.networks:
            networks.append(Network(network.name, NetworkType.PRIVATE,
                                    network.cidr))
        if border_router:
            networks.append(Network(config["border_router_network_name"],
                                    NetworkType.PRIVATE,
                                    config["border_router_network_ip"]))

        return networks

    @classmethod
    def _load_flavors(cls, flavors_path: Path) -> List[Flavor]:
        """Load list of possible flavors from configuration file"""

        flavors: List[Flavor] = []
        flavors_file_content: Dict = Reader.open_yaml(flavors_path)

        for flavor_name, attributes in flavors_file_content.items():
            try:
                memory: int = int(attributes["memory"])
            except ValueError:
                raise ValueError(f'Memory value "{attributes["memory"]}"'
                                 ' is not an integer')
            try:
                cpus: int = int(attributes["cores"])
            except ValueError:
                raise ValueError(f'Cpus value "{attributes["cores"]}"'
                                 ' is not an integer')
            flavors.append(Flavor(flavor_name, memory, cpus))

        return flavors

    @staticmethod
    def _controller_needed(devices: List[Device], ansible_installed: bool) \
            -> bool:
        """Check if a controller device should be created"""
        if ansible_installed:
            return False
        for device in devices:
            if device.protocol == Protocol.WINRM:
                return True
        return False

    @staticmethod
    def _resolve_flavor(flavor_name: str, flavors: List[Flavor]) -> Tuple[int, int]:
        for flavor in flavors:
            if flavor.name == flavor_name:
                return flavor.memory, flavor.cpus
        raise AttributeError(f"Invalid flavor: {flavor_name}")

    @staticmethod
    def _parse_memory_and_cpus(host: kypo.Host, flavors: List[Flavor])\
            -> Tuple[int, int]:
        if host.flavor:
            memory, cpus = Sandbox._resolve_flavor(host.flavor, flavors)
        if host.extra is not None:
            if "memory" in host.extra:
                try:
                    memory = int(host.extra["memory"])
                except ValueError:
                    raise ValueError(f'Memory value "{host.extra["memory"]}"'
                                     ' is not an integer')
            if "cpus" in host.extra:
                try:
                    cpus = int(host.extra["cpus"])
                except ValueError:
                    raise (f'Cpus value "{host.extra["cpus"]}" is not an'
                           'integer')
        return memory, cpus

    @staticmethod
    def _create_host_device(host: kypo.Host, topology: Topology,
                            flavors: List[Flavor], config: Dict,
                            networks: List[Network]) -> Device:

        memory, cpus = Sandbox._parse_memory_and_cpus(host, flavors)

        device_networks: List[Interface] = []
        for net_mapping in topology.net_mappings:
            if net_mapping.host == host.name:
                network = Sandbox._find_network(net_mapping.network,
                                                networks)
                device_networks.append(Interface(network, net_mapping.ip))

        if host.base_box.mgmt_protocol == kypo.Protocol.SSH:
            protocol: Protocol = Protocol.SSH
        elif host.base_box.mgmt_protocol == kypo.Protocol.WINRM:
            protocol: Protocol = Protocol.WINRM

        usb_passthrough: bool = False
        if host.extra is not None and "usb_passthrough" in host.extra:
            usb_passthrough: bool = host.extra["usb_passthrough"]

        return Device(host.name, DevicePurpose.HOST, host.base_box.image,
                      memory, cpus, protocol, device_networks, usb_passthrough)

    @staticmethod
    def _create_router_device(router: kypo.Router, topology: Topology,
                              flavors: List[Flavor], config: Dict,
                              networks: List[Network],
                              border_router: bool, router_n: int) -> Device:

        if router.flavor:
            memory, cpus = Sandbox._resolve_flavor(router.flavor, flavors)
        else:
            memory, cpus = config["default_router_memory"], \
                           config["default_router_cpus"]

        device_networks: List[Interface] = []
        for router_mapping in topology.router_mappings:
            if router_mapping.router == router.name:
                network = Sandbox._find_network(router_mapping.network,
                                                networks)
                device_networks.append(Interface(network, router_mapping.ip))
        if border_router:
            for net in networks:
                if net.name == config["border_router_network_name"]:
                    br_network: Network = net
            br_ip: IPAddress = IPAddress(config["border_router_ip"])
            device_networks.append(Interface(br_network, br_ip + router_n))

        if router.base_box.mgmt_protocol == kypo.Protocol.SSH:
            protocol: Protocol = Protocol.SSH
        elif router.base_box.mgmt_protocol == kypo.Protocol.WINRM:
            protocol: Protocol = Protocol.WINRM

        return Device(router.name, DevicePurpose.ROUTER, router.base_box.image,
                      memory, cpus, protocol, device_networks)

    @staticmethod
    def _find_network(network_name: str, networks: List[Network])\
            -> Optional[Network]:
        """Return a Network by its name"""
        for network in networks:
            if network.name == network_name:
                return network

    @staticmethod
    def _create_border_router(config: Dict, networks: List[Network]) -> Device:
        """Creates a border router device"""
        br_network: Interface =\
            Interface(Sandbox._find_network(
                config["border_router_network_name"], networks),
                config["border_router_ip"])

        return Device(config["border_router_name"], DevicePurpose.BORDER_ROUTER,
                      config["border_router_box"],
                      int(config["border_router_memory"]),
                      int(config["border_router_cpus"]), Protocol.SSH,
                      [br_network])

    @staticmethod
    def _find_available_ip(network: Network, devices: List[Device])\
            -> Interface:
        """Find an available IP address in a network and return the Interface"""

        all_addresses: List[IPAddress] = list(network.cidr)
        if len(all_addresses) > 5:  # avoid low addresses
            all_addresses = all_addresses[5:]
        new_ip: str = None
        for address in all_addresses:
            conflict = False
            for device in devices:
                for interface in device.interfaces:
                    if interface.network is network and interface.ip is address:
                        conflict = True
            if not conflict:
                new_ip = str(address)
                break
        return Interface(network, new_ip)

    @staticmethod
    def _create_device_list(topology: Topology, flavors: List[Flavor],
                            config: Dict, border_router: bool,
                            ansible_installed: bool,
                            networks: List[Network]) -> List[Device]:
        """Creates priority sorted list of devices with all attributes"""

        devices: List[Device] = []

        if border_router:
            devices.append(Sandbox._create_border_router(config, networks))

        for host in topology.hosts:
            devices.append(Sandbox._create_host_device(host, topology, flavors,
                                                       config, networks))
        router_n: int = 0
        for router in topology.routers:
            router_n += 1
            devices.append(Sandbox._create_router_device(router, topology,
                                                         flavors, config,
                                                         networks,
                                                         border_router, router_n))
        if Sandbox._controller_needed(devices, ansible_installed):  # needs to be at the end
            devices.append(Device(config["controller_name"],
                                  DevicePurpose.CONTROLLER,
                                  config["controller_box"],
                                  int(config["controller_memory"]),
                                  int(config["controller_cpus"]), Protocol.SSH,
                                  [Sandbox._find_available_ip(networks[0], devices)]))

        return sorted(devices)



from typing import Optional, List, Dict

from sandboxcreator.input_parser.sandbox import Network, Sandbox,\
    Interface, DevicePurpose, Device, DeviceType


class Routes:
    """Static methods for routing table changes"""

    @staticmethod
    def _find_router_interface(network: Network, sandbox: Sandbox) -> Interface:
        """Find the interface of a router inside a network"""
        for device in sandbox.devices:
            if device.device_purpose is DevicePurpose.ROUTER:
                for interface in device.interfaces:
                    if network == interface.network:
                        return interface
        raise AttributeError(f"There is no router in the network{network.name}")

    @staticmethod
    def _find_router_in_network(network: Network, sandbox: Sandbox) -> Device:
        """Find the a router inside a network"""
        for device in sandbox.devices:
            if device.device_purpose is DevicePurpose.ROUTER:
                for interface in device.interfaces:
                    if network == interface.network:
                        return device
        raise AttributeError(f"There is no router in the network{network.name}")

    @staticmethod
    def _find_br(sandbox: Sandbox) -> Optional[Device]:
        """Find border router"""
        for device in sandbox.devices:
            if device.device_purpose is DevicePurpose.BORDER_ROUTER:
                return device
        return None

    @staticmethod
    def _find_br_network(sandbox: Sandbox) -> Network:
        """Find the border router network"""
        for network in sandbox.networks:
            if network.name == sandbox.config["border_router_network_name"]:
                return network

    @staticmethod
    def _create_host_routes(device: Device, sandbox: Sandbox) -> List[Dict]:
        """Create routes for hosts (and controller)"""
        routes: List = []
        if sandbox.border_router_present:
            router_int: Interface = Routes. \
                _find_router_interface(device.interfaces[0].network,
                                       sandbox)
            to_router = {
                "interface_ip": str(device.interfaces[0].ip),
                "interface_netmask":
                    str(device.interfaces[0].network.cidr.netmask),
                "interface_default_gateway": str(router_int.ip),
                "interface_routes": []}
            routes.append(to_router)
        else:
            routes_to_add = []
            for network in sandbox.networks:
                if network is device.interfaces[0].network:
                    continue
                specific_route = {
                    "gateway": str(Routes._find_router_interface(
                        device.interfaces[0].network, sandbox).ip),
                    "network": str(network.cidr.ip),
                    "netmask": str(network.cidr.netmask)}
                routes_to_add.append(specific_route)
            routes.append({
                "interface_default_gateway": "",
                "interface_ip": str(device.interfaces[0].ip),
                "interface_netmask":
                    str(device.interfaces[0].network.cidr.netmask),
                "interface_routes": routes_to_add})
        return routes

    @staticmethod
    def _create_router_routes(device: Device, sandbox: Sandbox) -> List[Dict]:
        """Create routes for routers"""
        routes: List = []
        if sandbox.border_router_present:
            br: Device = Routes._find_br(sandbox)
            br_network: Network = Routes._find_br_network(sandbox)
            for interface in device.interfaces:
                if interface.network is br_network:
                    def_interface: Interface = interface
            to_br = {
                "interface_default_gateway":
                    str(sandbox.config["border_router_ip"]),
                "interface_ip": str(interface.ip),
                "interface_netmask": str(interface.network.cidr.netmask),
                "interface_routes": []}
            routes.append(to_br)
        return routes

    @staticmethod
    def _create_br_routes(device: Device, sandbox: Sandbox) -> List[Dict]:
        """Create routers the for border router"""
        routes: List = []
        br_network: Network = Routes._find_br_network(sandbox)
        for interface in device.interfaces:
            if interface.network is br_network:
                def_interface: Interface = interface
        to_other_networks: List = []
        for network in sandbox.networks:
            if network is br_network:
                continue
            router: Device = Routes._find_router_in_network(network, sandbox)
            for interface in router.interfaces:
                if interface.network == br_network:
                    interface_to_hosts = interface
            to_other_networks.append({"gateway": str(interface_to_hosts.ip),
                                      "network": str(network.cidr.ip),
                                      "netmask": str(network.cidr.netmask)})
        to_hosts = {
            "interface_default_gateway": "",
            "interface_ip": str(def_interface.ip),
            "interface_netmask": str(br_network.cidr.netmask),
            "interface_routes": to_other_networks}
        routes.append(to_hosts)
        return routes

    @staticmethod
    def create_routes(device: Device, sandbox: Sandbox) -> List[Dict]:
        """Create specific routes for each type of device"""
        routes: List[Dict] = []
        if sandbox.border_router_present and \
                device.device_purpose is not DevicePurpose.BORDER_ROUTER:
            auto_route: str = ""
        else:
            auto_route: str = "{{ ansible_default_ipv4.gateway }}"
        configure_auto = {
            "interface_ip": "{{ ansible_default_ipv4.address  | "
                            "default(ansible_all_ipv4_addresses[0]) }}",
            "interface_netmask": "{{ ansible_default_ipv4.netmask"
                                 "  | default('24') }}",
            "interface_default_gateway": auto_route}
        routes.append(configure_auto)

        if device.device_type is DeviceType.HOST:
            routes.extend(Routes._create_host_routes(device, sandbox))
        elif device.device_purpose is DevicePurpose.ROUTER:
            routes.extend(Routes._create_router_routes(device, sandbox))
        elif device.device_purpose is DevicePurpose.BORDER_ROUTER:
            routes.extend(Routes._create_br_routes(device, sandbox))

        return routes



#!/usr/bin/env python3

import argparse
import os
import sys
from typing import List

from sandboxcreator.creator import create


def _parse_cli_args(cli_args: List[str]) -> argparse.ArgumentParser:
    """Parse command line arguments"""

    parser = argparse.ArgumentParser()

    parser.add_argument("topology_file",
                        help="path to the topology definition")
    parser.add_argument("-o", "--output-dir",
                        help="output directory for the sandbox",
                        default="",
                        action="store")
    parser.add_argument("-a", "--ansible-installed",
                        help="use Ansible installed on the host machine",
                        action="store_true")
    parser.add_argument("-b", "--border-router",
                        help="create a border router",
                        action="store_true")
    parser.add_argument("--rewrite-provisioning",
                        help="always generate clean user provisioning",
                        action="store_true")
    parser.add_argument("--provisioning-dir",
                        help="path to directory with user provisioning files",
                        default="",
                        action="store")
    parser.add_argument("--extra-vars",
                        help="path to file with extra_vars for Ansible",
                        default="",
                        action="store")
    parser.add_argument("--verbose-ansible",
                        help="set verbose output for Ansible (-vv)",
                        action="store_true")

    return parser.parse_args(cli_args)


def _run_create(parsed_cli_args: argparse.ArgumentParser) -> None:
    """Run the create module to generate the intermediate definition"""

    try:
        create(parsed_cli_args.topology_file, parsed_cli_args.output_dir,
               parsed_cli_args.ansible_installed, parsed_cli_args.border_router,
               parsed_cli_args.provisioning_dir, parsed_cli_args.extra_vars,
               parsed_cli_args.rewrite_provisioning,
               parsed_cli_args.verbose_ansible, os.getcwd())
    except Exception as e:
        print(f"Could not create intermediate sandbox definition:\n{e}")
        sys.exit(1)

    print("Intermediate sandbox definition was successfully created")


def main():
    parsed_args: argparse.ArgumentParser = _parse_cli_args(sys.argv[1:])
    _run_create(parsed_args)


if __name__ == '__main__':
    main()



from pathlib import Path
from typing import Union, Optional

from sandboxcreator.input_parser.sandbox import Sandbox
from sandboxcreator.vagrant_generator.vagrantfile import Vagrantfile
from sandboxcreator.ansible_generator.preconfig import Preconfig
from sandboxcreator.ansible_generator.provisioning import Provision

CSC_PATH: Path = Path(__file__).parent.resolve()
CONFIGURATION_PATH: Path = CSC_PATH.joinpath("resources/conf/configuration.yml")
FLAVORS_PATH: Path = CSC_PATH.joinpath("resources/conf/flavors.yml")


def _process_working_dir(working_dir: Optional[Union[str, Path]]) \
        -> Optional[Path]:
    """Validate and resolve working directory"""
    if working_dir is None:
        return None

    try:
        if isinstance(working_dir, str):
            working_directory: Path = Path(working_dir)
        elif isinstance(working_dir, Path):
            working_directory: Path = working_dir
        else:
            raise TypeError("Working directory has unknown type "
                            f'"{type(working_dir)}"')
        if not working_directory.is_dir() or \
                not working_directory.is_absolute():
            raise ValueError("Working directory should be an absolute path")
    except OSError:
        raise ValueError(f'Invalid path to working directory "{working_dir}"')

    return working_directory


def _process_topology_path(topology: Union[str, Path],
                           working_dir: Optional[Path]) -> Path:
    """Validate and resolve topology path"""
    try:
        if isinstance(topology, str):
            topology_path: Path = Path(topology).resolve()
        elif isinstance(topology, Path):
            topology_path: Path = topology.resolve()
        else:
            raise TypeError("Topology definition file path has invalid type "
                            f"\"{type(topology)}\"")
        if working_dir is not None and not topology_path.is_absolute():
            topology_path = working_dir.joinpath(topology_path)
        if not topology_path.is_file():
            raise OSError(f"File \"{topology_path}\" does not exist")
    except OSError:
        raise ValueError(f"Invalid path to topology file \"{topology}\"")

    return topology_path


def _process_output_path(output_dir: Optional[Union[str, Path]],
                         topology_path: Path,
                         working_dir: Optional[Path]) -> Path:
    """Validate and resolve output directory path"""
    try:
        if output_dir is None or output_dir == "":
            sandbox_path: Path = topology_path.parent.joinpath("sandbox")
        elif isinstance(output_dir, str):
            sandbox_path: Path = Path(output_dir).resolve()
        elif isinstance(output_dir, Path):
            sandbox_path: Path = output_dir.resolve()
        else:
            raise TypeError("Output directory path has invalid type "
                            f"\"{type(output_dir)}\"")
        if working_dir is not None and not sandbox_path.is_absolute():
            sandbox_path = working_dir.joinpath(sandbox_path)
        if sandbox_path.is_file():
            raise OSError(f"\"{sandbox_path}\" is a file")
    except OSError:
        raise ValueError(f"Invalid output directory \"{output_dir}\"")

    return sandbox_path


def _process_provisioning_path(provisioning_dir: Optional[Union[str, Path]],
                               working_dir: Optional[Path]) -> Optional[Path]:
    """Validate and resolve user provisioning directory path"""
    try:
        if provisioning_dir is None or provisioning_dir == "":
            return None
        if isinstance(provisioning_dir, str):
            provisioning_path: Path = Path(provisioning_dir).resolve()
        elif isinstance(provisioning_dir, Path):
            provisioning_path: Path = provisioning_dir.resolve()
        else:
            raise TypeError("Provisioning directory path has invalid type "
                            f"\"{type(provisioning_dir)}\"")
        if working_dir is not None and not provisioning_path.is_absolute():
            provisioning_path = working_dir.joinpath(provisioning_path)
        if not provisioning_path.is_dir():
            raise ValueError(f"Directory \"{provisioning_path}\" does not exist")
        playbook_path: Path = provisioning_path.joinpath("playbook.yml")
        if not playbook_path.is_file():
            raise ValueError("Provisioning directory should contain "
                             "\"playbook.yml\"")
    except OSError:
        raise ValueError("Invalid path to provisioning directory "
                         f"\"{provisioning_dir}\"")

    return provisioning_path


def _process_extra_vars_path(extra_vars: Optional[Union[str, Path]],
                             working_dir: Optional[Path]) -> Optional[Path]:
    """Validate and resolve path to Ansible extra vars file"""
    try:
        if extra_vars is None or extra_vars == "":
            return None
        if isinstance(extra_vars, str):
            extra_vars_path: Path = Path(extra_vars).resolve()
        elif isinstance(extra_vars, Path):
            extra_vars_path: Path = extra_vars.resolve()
        else:
            raise TypeError("Extra vars file path has invalid type "
                            f"\"{type(extra_vars)}\"")
        if working_dir is not None and not extra_vars_path.is_absolute():
            extra_vars_path = working_dir.joinpath(extra_vars_path)
        if extra_vars_path is not None and not extra_vars_path.is_file():
            raise OSError(f"File \"{extra_vars_path}\" does not exist")
    except OSError:
        raise ValueError("Invalid path to extra variables file "
                         f"\"{extra_vars}\"")

    return extra_vars_path


def _generate_vagrantfile(sandbox_definition: Sandbox) -> None:
    """Initiate Vagrantfile generation"""
    try:
        vagrant_definition: Vagrantfile = Vagrantfile(sandbox_definition)
        vagrant_definition.generate(sandbox_definition.sandbox_dir /
                                    "Vagrantfile", "vagrantfile.j2")
    except IOError as e:
        raise RuntimeError(f"Vagrantfile could not be generated:\n{e}")


def _generate_preconfig(sandbox_definition: Sandbox) -> None:
    """Initiate preconfig directory generation"""
    try:
        Preconfig.generate_preconfig(sandbox_definition)
    except IOError as e:
        raise RuntimeError(f"Provisioning files could not be generated:\n{e}")


def _generate_provisioning(sandbox_definition: Sandbox) -> None:
    """Initiate provisioning directory generation"""
    try:
        Provision.generate_user_provisioning(sandbox_definition)
    except IOError as e:
        raise RuntimeError("User provisioning files could not be generated:"
                           f"\n{e}")


def create(topology: Union[str, Path],
           output_dir: Optional[Union[str, Path]] = None,
           ansible_installed: bool = False, border_router: bool = False,
           provisioning_dir: Optional[Union[str, Path]] = None,
           extra_vars: Optional[Union[str, Path]] = None,
           generate_provisioning: bool = False,
           verbose_ansible: bool = False,
           working_dir: Optional[Union[str, Path]] = None) -> None:
    """Generates intermediate definition from a topology definition.

    :param topology: path to the topology definition file
    :param output_dir: path to an output directory
    :param ansible_installed: whether the host machine have ansible installed
    :param border_router: whether border router should be created
    :param provisioning_dir: path to directory with user provisioning files
    :param extra_vars: path to YAML file with additional values for Ansible
    :param generate_provisioning: whether provisioning should be regenerated
    :param verbose_ansible: whether Ansible output should be set to verbose
    :param working_dir: an absolute path to resolve possible relative paths
    """

    working_path: Optional[Path] = _process_working_dir(working_dir)
    topology_path: Path = _process_topology_path(topology, working_path)
    output_path: Path = _process_output_path(output_dir, topology_path,
                                             working_path)
    provisioning_path: Path = _process_provisioning_path(provisioning_dir,
                                                         working_path)
    extra_vars_path: Path = _process_extra_vars_path(extra_vars, working_path)

    try:
        sandbox_definition: Sandbox = Sandbox(topology_path, CONFIGURATION_PATH,
                                              FLAVORS_PATH, output_path,
                                              border_router, ansible_installed,
                                              provisioning_path,
                                              extra_vars_path,
                                              generate_provisioning,
                                              verbose_ansible)
    except Exception as e:
        raise RuntimeError(f"Definition parsing has failed:\n{e}")

    _generate_vagrantfile(sandbox_definition)
    _generate_preconfig(sandbox_definition)
    _generate_provisioning(sandbox_definition)



import os
import yaml

from pathlib import Path
from typing import Union, List, Dict


class Reader:
    """Static methods for reading from files"""

    @staticmethod
    def open_yaml(file_path: Path) -> Union[List, Dict]:
        """Open a yaml file and return its content"""
        try:
            with file_path.open() as input_file:
                return yaml.safe_load(input_file)
        except yaml.YAMLError:
            raise IOError(f"Could not parse yaml file {file_path}.")
        except IOError:
            raise IOError(f"Could not open yaml file {file_path}.")

    @staticmethod
    def file_exists(file: Path) -> bool:
        """Check if the given file exists

        :param file: path to a file
        """

        return os.path.isfile(file)

    @staticmethod
    def dir_exists(directory: Path) -> bool:
        """Check if the given directory exists

        :param directory: path to a directory
        """

        return os.path.isdir(directory)



import shutil
import jinja2
import os
import yaml
from pathlib import Path
from typing import Union, List, Dict

from yaml.emitter import Emitter
from yaml.representer import Representer
from yaml.resolver import Resolver
from yaml.serializer import Serializer


class Writer:
    """Static methods for generating files"""

    TEMPLATE_DIR: Path = Path(__file__).parent.parent / "resources/templates"

    @staticmethod
    def _write_to_file(filepath: Path, content: str) -> None:
        """Generate a file from output string."""
        try:
            os.makedirs(os.path.dirname(filepath), exist_ok=True)
            with open(filepath, "w") as new_file:
                new_file.write(content)
        except IOError:
            raise IOError(f"Could not create file {filepath}")

    @staticmethod
    def generate_yaml(output_file: Path, data: Union[List, Dict]) -> None:
        """Generate YAML file from a List or Dict

        :param output_file: path to the output file
        :param data: List or a Dict with data for the YAML file
        """

        Writer._write_to_file(output_file, yaml.dump(data, explicit_start=True,
                                                     explicit_end=True))

    @staticmethod
    def generate_file_from_template(output_file: Path, template_name: str,
                                    data=None, ruby_variables=None) -> None:
        """Generate file from Jinja2 template

        :param output_file: path to the output file
        :param template_name: name of a Jinja2 template file in templates dir
        :param data: arbitrary number of named args for the template
        :param ruby_variables: additional variables to pass to the template
        """

        try:
            template_loader = jinja2.FileSystemLoader(searchpath=
                                                      str(Writer.TEMPLATE_DIR))
            template_env = jinja2.Environment(loader=template_loader,
                                              trim_blocks=True,
                                              lstrip_blocks=True)
            template = template_env.get_template(template_name)
        except jinja2.TemplateNotFound:
            raise RuntimeError(f"Could not find template {template_name}")

        output = template.render(data=data, ruby_variables=ruby_variables)
        Writer._write_to_file(output_file, output)

    @staticmethod
    def clone_git_repository(repository: str, location: Path) -> None:
        """Clone a git repository to the provided location

        :param repository: repository location
        :param location: target directory
        """

        try:
            os.makedirs(location, exist_ok=True)
            os.chdir(location)
            os.system(f"git clone -q {repository}")
        except RuntimeError:
            raise RuntimeError(f"Could not clone repository: {repository}")

    @staticmethod
    def copy_file(source: Path, destination: Path):
        """Copies a file from the source path to the destination

        :param source: path to the source file
        :param destination: path to the destination file
        """

        try:
            os.makedirs(destination.parent, exist_ok=True)
            shutil.copyfile(source, destination)
        except OSError:
            raise RuntimeError(f"Could not copy file {source}")

    @staticmethod
    def copy_directory(source: Path, destination: Path):
        """Copy directory recursively form the source dir to the destination.

        :param source: path to source directory
        :param destination: path to the destination directory
        """

        try:
            os.makedirs(destination.parent, exist_ok=True)
            shutil.copytree(source, destination)
        except OSError:
            raise RuntimeError(f"Could not copy directory {source}")

    @staticmethod
    def remove_directory(directory: Path):
        """Remove the directory recursively

        :param directory: path to directory to remove
        """

        try:
            shutil.rmtree(directory)
        except OSError:
            raise RuntimeError(f"Could not remove directory {directory}")
